import com.jinsight.jetchart.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.applet.*;
import java.util.StringTokenizer;
import java.io.*;
import java.net.*;
import java.util.Vector;

/**
 *
 * This demo displays a historical sequence of monthly Margin Debt data, spanning
 * from 1931 to 2000, plotted  against  a  logarithmic scale and showing a linear
 * regression line. Data comes from an external xml file.
 * Data Source: Financial Markets Center - http://www.fmcenter.org
 *	
 * @author Andre de Lima Soares - alsoares@jinsight.com 
 */
public class Main extends JFrame implements ActionListener,ItemListener {
    
    Graph graph;
    LineSerie ls;
    TargetLine tl;
    int[] status;

    JCheckBox isLogarithmic,showInnerGridLines;

    public Main() {
	
        setTitle("The JetChart Library - Logarithmic scale demo");

        addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent evt) {
              System.exit(0);
           }
        });

	// Creates a top panel.
        createTopPanel();

	// Creates the chart context.
        createChartContext();

	// Adjusts scale properties.
        setScaleProperties();

        // Creates target line to display text at a fixed position.
	createTargetLine();

	// Enables grid.
        setGridProperties();
	
        // Create line series.
        createLineSeries();

        // Adds series to the chart context.
	graph.addSerie(ls);

        Container ct=getContentPane();

	ct.add("Center",graph);
       
	setSize(550,500);
	
	setVisible(true);
	
    }
    
    private void createTopPanel() {

        // Creates a top panel and adds a button to it. The
        // button is used to load chart data from an xml file.
	JPanel topPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
	
	JButton b=new JButton("Load data");
	b.addActionListener(this);
	topPanel.add(b);

	// Creates a check box to control scale style(log or linear).
        isLogarithmic=new JCheckBox("Logarithmic");
        isLogarithmic.setSelected(true);
        isLogarithmic.addItemListener(this);
	isLogarithmic.setEnabled(false);

        topPanel.add(isLogarithmic);

	// Creates a check box to control the visibility of the inner grid lines.
        showInnerGridLines=new JCheckBox("Show inner grid lines");
        showInnerGridLines.setSelected(true);
        showInnerGridLines.addItemListener(this);
	showInnerGridLines.setEnabled(false);
        topPanel.add(showInnerGridLines);
	
        Container ct=getContentPane();

	ct.add("North",topPanel);

    }

    private void createChartContext() {

       	graph=new Graph();

        // Sets chart title and its font.
	graph.setTitle(new String[]{"Margin Debt: 1931-2000","Logarithmic Scale"});
	graph.setTitleFont(new Font("Dialog",Font.PLAIN,14));

        // Disables automatic label spacing, since labels visibility will be manually set.
	graph.setAutoLabelSpacingEnabled(false);

        // Sets labels font.
	graph.setLabelsFont(new Font("SansSerif",Font.PLAIN,10));

        // Enables vertical labels.
	graph.setVerticalLabelsEnabled(true);

        // Enables the scrolling label, so we can see labels scrolling on the
        // chart top while moving mouse cursor across the chart area.
	graph.getScrollingLabel().setEnabled(true);

	// Enables tooltip display. If multiple tooltips display is enabled,
	// they are centered in the chart area.
	graph.getToolTip().setEnabled(true);
	graph.getToolTip().setMultipleEnabled(true);
	graph.getToolTip().setPointerType(ToolTip.NONE);

	// Sets tooltip value format.
	graph.setValueFormat("#,##0.00");
	
        // Enables the scrolling bar.
        GraphSet graphSet=graph.getGraphSet(0);
        ScrollingBar sb=graphSet.getScrollingBar();
	sb.setEnabled(true);

        // Series can be highlighted at the same time while moving mouse cursor
        // over the chart area. We just have one series but we want to see data
        // points highlighted without having to move cursor to the coordinate of
        // a data point. To accomplish this, we just enable simultaneous highlight 
        // of multiple series.
	graph.setMultipleHighlightEnabled(true);

        // Sets the left title text.
	graph.getLeftTitle().setText("Billions");

    }

    private void setScaleProperties() {
       
        GraphSet graphSet=graph.getGraphSet(0);
	Scale scale=graphSet.getScale();
	scale.setValueFormat("$ ###,###,###");

        // Enables a logarithmic scale.
	scale.setLogEnabled(true);

        // Sets the logarithimic scale base.
	scale.setLogBase(10);

        // Disable the automatic scale. A logarithmic scale is only
        // displayed  if scale  is  not  automatically  calculated.
	scale.setAutoScaleEnabled(false);

        // Sets scale maximum value.
	scale.setMaxValue(1000000);

	// Sets the only scale labels to be displayed. The values passed
	// to the method below are the labels indexes, counted from top
	// to bottom.
        scale.setVisibleLabelsIndexes(new int[]{0,9,18,27,36,45,54});

    }
 

    private void createTargetLine() {

        // Creates a target line to display a text below the line series.
        tl=new TargetLine(100);
	tl.setText("Margin Debt has grown unusually fast since 1994.");

        // Sets the target line color to be equal to the grid line color.
        tl.setColor(Color.lightGray);

        GraphSet graphSet=graph.getGraphSet(0);
	graphSet.addTargetLine(tl);

    } 
	
    private void setGridProperties() {

        GraphSet graphSet=graph.getGraphSet(0);

	// Enables the grid and sets its color.
       	Grid grid=graphSet.getGrid();
	grid.setEnabled(true);
	grid.setColor(Color.lightGray);

	// Enables the inner logarithmic lines.
	grid.setInnerLogLinesEnabled(true);
	
	// Sets the number of inner lines between two consecutive
	// labels.
	grid.setInnerLogLinesCount(8);
	
    }

    private void createLineSeries() {

       	// Creates line series and sets its id. The xml parser identifies a series
	// by its id number.
	ls=new LineSerie();
	ls.setId(0);
	
	ls.setTitle("Margin Debt");
	ls.setColor(Color.red);
	ls.setThickness(2);
	ls.setMarksEnabled(false);

       	// Creates a blue linear regression line.
	RegressionLine rl=ls.getRegressionLine();
	rl.setEnabled(true);
	rl.setColor(Color.blue);
	
	// Sets the intercept value. It corresponds to the point where the
	// regression line cuts the vertical axis. This configuration will
	// be used to change the position of the regression line upwards,
	// if scale is changed to be linear. This method only has effect if
	// the method RegressionLine.setAutoInterceptEnabled() is set to 
	// false. See the itemStateChanged handler.
	rl.setInterceptValue(25000);

    }

    public void actionPerformed(ActionEvent evt) {
	
	// Reads labels and values from the chart.xml file.
	try {
	    FileReader fr=new FileReader("chart.xml");
	    graph.readXMLData(fr);

            // Gets labels from the chart context, in order to set labels visibility.
	    String[] labels=graph.getLabels();

	    // Sets labels status. Only even years are displayed.
	    graph.setLabelsStatus(getStatus(labels));

	    // Enables check box to alternate between logarithmic and linear scale.
	    isLogarithmic.setEnabled(true);

	    // Enables check box to control the inner grid lines visibility.
	    showInnerGridLines.setEnabled(true);

	    graph.repaint();
	}
	catch (IOException e) {
	    e.printStackTrace();
	}
	
    }
    
    public void itemStateChanged(ItemEvent evt) {

        GraphSet graphSet=graph.getGraphSet(0);
        Scale scale=graphSet.getScale();
        Grid grid=graphSet.getGrid();
	
	// Gets the states of the Checkbox objects.
	boolean isLog=isLogarithmic.isSelected();
	boolean showInnerGrid=showInnerGridLines.isSelected();
	
	if (evt.getSource()==isLogarithmic) {
	    
	    // Enables/disables log scale.
	    scale.setLogEnabled(isLog);

	    // Enables/disables check box that controls inner grid lines visibility.
	    showInnerGridLines.setEnabled(isLog);
	    
	    // If scale is logarithmic...
	    if (isLog) {
		
		// If the inner grid lines are visible...
		if (showInnerGrid)
		    // The labels corresponding to the inner grid lines are not displayed.
		    scale.setVisibleLabelsIndexes(new int[]{0,9,18,27,36,45,54});
		else
		    // All labels are displayed.
		    scale.setVisibleLabelsIndexes(null);
		
		// Sets the maximum value to 1,000,000.
		scale.setMaxValue(1000000);
		
		// Adjusts the position of the target line.
		tl.setValue(100);
		
		// The regression line intercept point is automatic calculated.
		ls.getRegressionLine().setAutoInterceptEnabled(true);
	    }
	    else { 
		// Sets the maximum value to 300,000.
		scale.setMaxValue(300000);
		// Sets scale increment to 50,000.
		scale.setIncrement(50000);
		// All labels are displayed.
		scale.setVisibleLabelsIndexes(null);
		tl.setValue(150000);
		
		// Disables the automatic calculation of the intercept point. 
		ls.getRegressionLine().setAutoInterceptEnabled(false);

	    }
	    
	}
	else {

	    // Enables/disables display of the inner grid lines.
	    grid.setInnerLogLinesEnabled(showInnerGrid);
	    if (showInnerGrid) 
		// If enabled, all labels corresponding to the inner lines are disabled.
		scale.setVisibleLabelsIndexes(new int[]{0,9,18,27,36,45,54});
	    else
		// All labels are displayed.
		scale.setVisibleLabelsIndexes(null);
	}
	
	graph.repaint();
	
    }

    private int[] getStatus(String[] labels) {
	
        // Creates an array of integer values to set labels status. 
	int[] status=new int[labels.length];
	
        // Loops across the labels array.
	for (int counter=0;counter<labels.length;counter++) {
	    String label=labels[counter];

            /**
             * Labels content type varies between months and years.
             * We only want even years displayed, so we set the respective labels
             * status accordingly(1 - visible).
             */
	    if (label.startsWith("19") || label.startsWith("20")) {

		int year=Integer.parseInt(label);

		if ((year/2)*2==year) 
		    status[counter]=1;
		
	    }

	}
	
	return status;

    }
    
    public static void main(String[] args) {
	Main main=new Main();
    }

}
                

