import com.jinsight.jetchart.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * This application  demonstrates how  to  combine  graph  zooming  with  scrolling  of
 * zoomed areas. JetChart  dispatches  a  zooming event  each  time  chart is zoomed-in
 * or zoomed-out. By capturing these events the application is able to set the properties
 * of a scrolling component, making it possible to adjust the set of visible data points 
 * while sliding the scrolling bar. The flexibility  achieved with this simple model is
 * far better  than having  the  scrolling mechanism  implemented in the class library,
 * since any scrolling component can be used.
 * In order to capture zoom events, the  application  must adhere to  the ZoomListener
 * interface, implementing only one method(ZoomListener.chartZoomed(ZoomEvent evt)).
 *
 * @author Andre de Lima Soares - alsoares@jinsight.com
 */
public class Main extends JFrame implements AdjustmentListener,ZoomListener {

    JScrollBar scrollBar;

    Graph graph;

    double[]  values;

    public Main() {

	setTitle("The JetChart Library - zoom demo");

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    System.exit(0);
		}
	    });
	
	// Creates the chart context.
	graph=new Graph();

	// Enables zoom.
	graph.setZoomEnabled(true);

        // Zoom events are captured by this application.
	graph.addZoomListener(this);

	graph.set3DEnabled(true);

	// Enables the grid and crossed grid lines of the primary GraphSet object.
	// See JetChart documentation about the GraphSet class for further details.
	Grid grid=graph.getGraphSet(0).getGrid();
	grid.setEnabled(true);
	GridCrossedLines gridCrossedLines=graph.getGraphSet(0).getGridCrossedLines();
	gridCrossedLines.setEnabled(true);
	gridCrossedLines.setColor(Color.cyan);

	// Enables the right y axis.
	Scale scale=graph.getGraphSet(0).getScale();
	scale.setPosition(Scale.LEFT_RIGHT);

	// Disables automatic scale and set increment and max values.
	scale.setAutoScaleEnabled(false);
	scale.setMaxValue(150);
	scale.setIncrement(20);

	graph.getToolTip().setEnabled(true);
	
	graph.setStartDate("07012001");

	// Values for the area series.
	values=new double[]{131.90,132.80,130.50,131.00,136.75,135.00,131.50,130.50,
			    132.40,133.30,130.90,123.40,122.25,118.00,120.00,116.10,
			    115.00,103.45,107.50,103.80,99.00,96.75,94.00,95.90,97.50,
			    98.40,100.20,102.00,110.40,104.50,108.70,107.75,107.45,
			    103.00,100.90,99.25,93.80,99.25,103.00,100.25,102.70,
			    107.15,106.00,107.15,109.00,104.00,96.50,97.95,99.70,
			    97.70,91.00,87.45,86.80,88.60,85.40,85.90,84.85,
			    76.55, 68.00,67.45,61.90,60.90,60.75,53.60,58.50,
			    57.20,51.25,48.50,45.75,46.90,52.55,51.40,54.00,
			    51.70,41.20,38.75,34.00,33.50,33.25,34.25,33.90,
			    30.00,33.50,34.95,31.45,26.95,28.40,30.90,28.90,
			    29.90,29.80,33.25,34.50,34.00,36.90,37.00,36.40,
			    34.70,37.50,38.30,39.40,43.50,45.80,48.70,46.45};

	// Creates a scrolling bar and adds it to a bottom panel.
	scrollBar=new JScrollBar(JScrollBar.HORIZONTAL,0,0,0,0);
	scrollBar.addAdjustmentListener(this);

	JPanel bottomPanel=new JPanel(new BorderLayout());
	
	bottomPanel.add(scrollBar,BorderLayout.CENTER);

        Container ct=getContentPane();

	ct.add("South",bottomPanel);


	// Creates an area series.
	AreaSerie as=new AreaSerie(values,"Area Series");
	as.setColor(Color.red);
	as.setAreaLinesEnabled(false);
	
	// Adds the line series to the chart context.
	graph.addSerie(as);

	ct.add(graph,BorderLayout.CENTER);

	setSize(600,400);
	
	setVisible(true);

    }

    // Implements the ZoomListener interface. Each time chart is zoomed,
    // a ZoomEvent object is dispatched to this method.
    public void chartZoomed(ZoomEvent evt) {

        // A ZoomEvent  object  carries information  concerning the maximum
        // number of data points, the  selected  number  of data points and
        // the first  value  in the sequence of selected data points. These
        // three values are used to configure the properties of a scrolling
        // bar. 
        if (evt.getZoomType()==ZoomEvent.ZOOM_IN) { // If chart is zoomed-in...
            scrollBar.setMaximum(evt.getMaximum());
            scrollBar.setVisibleAmount(evt.getVisibleAmount());
            scrollBar.setValue(evt.getValue());
        }
        else if (evt.getZoomType()==ZoomEvent.ZOOM_OUT) { // If chart is zoomed-out... 
            scrollBar.setMaximum(0);
            scrollBar.setVisibleAmount(0);
            scrollBar.setValue(0);
        }

    }

    // Implements the AdjustmentListener interface. While sliding a scrolling
    // bar, an  AdjustmentEvent  is  dispatched to this method. The scrolling
    // bar properties are captured and assigned to the setDataSetRange method.
    public void adjustmentValueChanged(AdjustmentEvent evt) {

	int startIndex=evt.getValue();
	int endIndex=evt.getValue()+scrollBar.getVisibleAmount()-1;

        // This  method  is used  to shrink the  range  of visible data points,
        // by setting the first and last range indexes. Any data points outside
        // of this range are not displayed.
	graph.setDataSetRange(startIndex,endIndex);

	graph.repaint();

    }

   
    public static void main(String[] args) {
	new Main();
    }
} 
	
